@@ -34,23 +34,58 @@ class Machine(object):
     """
     A local proxy representing the state of a remote CloudAPI machine.
     
-    A Machine object is intended to be a convenient container for methods and 
-    data relevant to a remotely running compute node managed by CloudAPI. A 
-    Machine is tied to a DataCenter object, and makes all its requests via 
-    that interface. It does not attempt to manage the state cache in most 
-    cases, instead requiring the user to explicitly update with a refresh() 
-    call.
+    A :py:class:`smartdc.machine.Machine` object is intended to be a 
+    convenient container for methods and data relevant to a remotely running 
+    compute node managed by CloudAPI. A :py:class:`smartdc.machine.Machine` is 
+    tied to a :py:class:`smartdc.datacenter.DataCenter` object, and makes all 
+    its requests via that interface. It does not attempt to manage the state 
+    cache in most cases, instead requiring the user to explicitly update with 
+    a :py:meth:`refresh` call.
     """
     def __init__(self, datacenter, machine_id=None, data=None):
         """
-        Typically, a Machine object is instantiated automatically by a
-        DataCenter object, but a user may instantiate one with a minimum
-        of a 'datacenter' object and a unique ID according to the machine. 
-        If data is passed in to instantiate, then the init method takes 
-        in the dict and populates its internal values from that.
+        :param datacenter: datacenter that contains this machine
+        :type datacenter: :py:class:`smartdc.datacenter.DataCenter`
+        
+        :param machine_id: unique ID of the machine
+        :type machine_id: :py:class:`basestring`
+        
+        :param data: raw data for instantiation
+        :type data: :py:class:`dict`
+        
+        Typically, a :py:class:`smartdc.machine.Machine` object is 
+        instantiated automatically by a 
+        :py:class:`smartdc.datacenter.DataCenter` object, but a user may 
+        instantiate one with a minimum of a `datacenter` parameter and a 
+        unique ID according to the machine. The object then pulls in the
+        machine data from the datacenter API. If `data` is passed in to  
+        instantiate, then ingest the dict and populate internal values from 
+        that.
+        
+        All of the following attributes are read-only:
+        
+        :var name: human-readable label for the machine
+        :var id: identifier for the machine
+        :var type: type (`smartmachine` or `virtualmachine`) of the 
+            machine
+        :var state: last-known state of the machine
+        :var dataset: the machine template
+        :var memory: the RAM (MiB) allocated for the machine 
+            (:py:class:`int`\)
+        :var disk: the persistent storage (MiB) allocated for the 
+            machine (:py:class:`int`\)
+        :var ips: :py:class:`list` of IPv4 addresses for the machine
+        :var metadata: :py:class:`dict` of user-generated attributes for 
+            the machine
+        :var created: :py:class:`datetime.datetime` of machine creation 
+            time
+        :var updated: :py:class:`datetime.datetime` of machine update 
+            time
         """
         self.id = machine_id or data.pop('id')
         self.datacenter = datacenter
+        """the :py:class:`smartdc.datacenter.DataCenter` object that holds 
+        this machine"""
         if not data:
             data = self.datacenter.raw_machine_data(self.id)
         self._save(data)
@@ -88,16 +123,6 @@ class Machine(object):
         self.created = dt_time(data.get('created'))
         self.updated = dt_time(data.get('updated'))
     
-    def refresh(self):
-        """
-        GET /:login/machines/:id
-        
-        Fetch the existing state and values for the Machine from the 
-        DataCenter and commit the values locally.
-        """
-        data = self.datacenter.raw_machine_data(self.id)
-        self._save(data)
-    
     @property
     def path(self):
         """
@@ -122,9 +147,63 @@ class Machine(object):
         """
         return filter(priv, self.ips)
     
+    @classmethod
+    def create_in_datacenter(cls, datacenter, **kwargs):
+        """
+        ::
+        
+            POST /:login/machines
+        
+        Class method, provided as a convenience. 
+        
+        :param datacenter: datacenter for creating the machine
+        :type datacenter: :py:class:`smartdc.datacenter.DataCenter`
+        
+        Provision a machine in the current 
+        :py:class:`smartdc.datacenter.DataCenter`, returning an 
+        instantiated :py:class:`smartdc.machine.Machine` object. All of the 
+        parameter values are optional, as they are assigned default values by 
+        the Datacenter's API itself. 
+        
+        'datacenter' is the only required argument. The rest of the arguments 
+        are passed to the DataCenter object as with 
+        :py:meth:`smartdc.datacenter.DataCenter.create_machine`.
+        """
+        return datacenter.create_machine(**kwargs)
+    
+    def refresh(self):
+        """
+        ::
+        
+            GET /:login/machines/:id
+        
+        Fetch the existing state and values for the 
+        :py:class:`smartdc.machine.Machine` from the datacenter and commit the 
+        values locally.
+        """
+        data = self.datacenter.raw_machine_data(self.id)
+        self._save(data)
+    
+    def status(self):
+        """
+        ::
+        
+            GET /:login/machines/:id
+        
+        :Returns: the current machine state
+        :rtype: :py:class:`basestring`
+        
+        Refresh the machine's information by fetching it remotely, then 
+        returning the :py:attr:`state` as a string.
+        """
+        self.refresh()
+        return self.state
+    
     def stop(self):
         """
-        POST /:login/machines/:id?action=stop
+        ::
+        
+            POST /:login/machines/:id?action=stop
         
         Initiate shutdown of the remote machine.
         """
@@ -134,7 +213,9 @@ class Machine(object):
     
     def start(self):
         """
-        POST /:login/machines/:id?action=start
+        ::
+        
+            POST /:login/machines/:id?action=start
         
         Initiate boot of the remote machine.
         """
@@ -144,7 +225,9 @@ class Machine(object):
     
     def reboot(self):
         """
-        POST /:login/machines/:id?action=reboot
+        ::
+        
+            POST /:login/machines/:id?action=reboot
         
         Initiate reboot of the remote machine.
         """
@@ -154,7 +237,9 @@ class Machine(object):
     
     def resize(self, package):
         """
-        POST /:login/machines/:id?action=resize
+        ::
+        
+            POST /:login/machines/:id?action=resize
         
         Initiate resizing of the remote machine to a new package.
         """
@@ -165,96 +250,75 @@ class Machine(object):
         j, r = self.datacenter.request('POST', self.path, params=action)
         r.raise_for_status()
     
-    def status(self):
-        """
-        GET /:login/machines/:id
-        
-        Refresh the machine's information by fetching it remotely, then 
-        returning the state as a string.
-        """
-        self.refresh()
-        return self.state
-    
     def delete(self):
         """
-        DELETE /:login/machines/:id
+        ::
+        
+            DELETE /:login/machines/:id
         
         Initiate deletion of a stopped remote machine.
         """
         j, r = self.datacenter.request('DELETE', self.path)
         r.raise_for_status()
     
-    @classmethod
-    def create_in_datacenter(cls, datacenter, **kwargs):
-        """
-        POST /:login/machines
-        
-        Class method, provided as a convenience. 
-        Provision a machine in the current DataCenter, returning an 
-        instantiated Machine() object. All of the parameter values are 
-        optional, as they are assigned default values by the Datacenter's API 
-        itself. 
-        
-        'datacenter' is the only required argument. The rest of the arguments 
-        are passed to the DataCenter object as with 
-        DataCenter.create_machine():
-        
-        'name' is a string used as a humab-readable label for the machine.
-        
-        'package' is a cluster of resource values identified by name. If 
-        passed a dict containing a 'name' key, it uses the corresponding 
-        value.
-        
-        'dataset' is base OS image identified by a globally unique ID or URN. 
-        If passed a dict containing an 'urn' or 'id' key, it uses the 
-        corresponding value. The server API appears to resolve incomplete or
-        ambiguous URNs with the highest version number.
-        
-        'metadata_dict' and 'tag_dict' are optionally passed dicts containing
-        arbitrary key-value pairs. A guideline for determining between the two 
-        is that tags may be used as filters when querying for and listing 
-        machines, while a metadata dict is returned when requesting details of 
-        a machine.
-        """
-        return datacenter.create_machine(**kwargs)
-    
     def poll_until(self, state, interval=2):
         """
-        GET /:login/machines/:id
+        ::
+        
+            GET /:login/machines/:id
+        
+        :param state: target state
+        :type state: :py:class:`basestring`
+        
+        :param interval: pause in seconds between polls
+        :type interval: :py:class:`int`
         
         Convenience method that continuously polls the current state of the 
-        machine remotely, and returns until the named 'state' argument is 
-        reached. The default wait 'interval' between requests is 2 seconds, 
+        machine remotely, and returns until the named `state` argument is 
+        reached. The default wait `interval` between requests is 2 seconds, 
         but it may be changed.
         
-        Note that if the next state is wrongly identified, this method may
-        loop forever.
+        .. Note:: If the next state is wrongly identified, this method may
+            loop forever.
         """
         while self.status() != state:
             time.sleep(interval)
     
     def poll_while(self, state, interval=2):
         """
-        GET /:login/machines/:id
+        ::
+        
+            GET /:login/machines/:id
+        
+        :param state: (assumed) current state
+        :type state: :py:class:`basestring`
+        
+        :param interval: pause in seconds between polls
+        :type interval: :py:class:`int`
         
         Convenience method that continuously polls the current state of the 
-        machine remotely, and returns while the machine has the named 'state' 
+        machine remotely, and returns while the machine has the named `state` 
         argument. Once the state changes, the method returns. The default wait 
-        'interval' between requests is 2 seconds, but it may be changed.
+        `interval` between requests is 2 seconds, but it may be changed.
         
-        Note that if a state transition has not correctly been triggered, this
-        method may loop forever.
+        .. Note:: If a state transition has not correctly been triggered, this
+            method may loop forever.
         """
         while self.status() == state:
             time.sleep(interval)
     
     def get_metadata(self):
         """
-        GET /:login/machines/:id/metadata
+        ::
+        
+            GET /:login/machines/:id/metadata
+        
+        :Returns: machine metadata
+        :rtype: :py:class:`dict`
         
         Fetch and return the metadata dict for the machine. The method 
         refreshes the locally cached copy of the metadata kept in the 
-        'metadata' attribute and returns it.
+        :py:attr:`metadata` attribute and returns it.
         """
         j, _ = self.datacenter.request('GET', self.path + '/metadata')
         self.metadata = j
@@ -262,11 +326,17 @@ class Machine(object):
     
     def update_metadata(self, **kwargs):
         """
-        POST /:login/machines/:id/metadata
+        ::
+        
+            POST /:login/machines/:id/metadata
+        
+        :Returns: current metadata
+        :rtype: :py:class:`dict`
         
         Send an metadata dict update for the machine (following dict.update() 
-        semantics). The method also refreshes the locally cached copy of the 
-        metadata kept in the 'metadata' attribute and returns it.
+        semantics) using the keys and values passed in the keyword arguments. 
+        The method also refreshes the locally cached copy of the metadata kept 
+        in the :py:attr:`metadata` attribute and returns it.
         """
         j, _ = self.datacenter.request('POST', self.path + '/metadata', 
                     params=kwargs)
@@ -275,10 +345,18 @@ class Machine(object):
     
     def delete_metadata_at_key(self, key):
         """
-        DELETE /:login/machines/:id/metadata/:key
+        ::
         
-        Deletes the Machine metadata contained at 'key'. Also explicitly 
-        requests and returns the Machine metadata so that the local copy stays 
+            DELETE /:login/machines/:id/metadata/:key
+        
+        :param key: identifier for matadata entry
+        :type key: :py:class:`basestring`
+        
+        :Returns: current metadata
+        :rtype: :py:class:`dict`
+        
+        Deletes the machine metadata contained at 'key'. Also explicitly 
+        requests and returns the machine metadata so that the local copy stays 
         synchronized.
         """
         j, r = self.datacenter.request('DELETE', self.path + '/metadata/' + 
@@ -288,10 +366,15 @@ class Machine(object):
     
     def delete_all_metadata(self):
         """
-        DELETE /:login/machines/:id/metadata
+        ::
         
-        Deletes all the metadata stored for this Machine. Also explicitly 
-        requests and returns the Machine metadata so that the local copy stays 
+            DELETE /:login/machines/:id/metadata
+        
+        :Returns: current metadata
+        :rtype: empty :py:class:`dict`
+        
+        Deletes all the metadata stored for this machine. Also explicitly 
+        requests and returns the machine metadata so that the local copy stays 
         synchronized.
         """
         j, r = self.datacenter.request('DELETE', self.path + '/metadata')
@@ -300,20 +383,26 @@ class Machine(object):
     
     def get_tags(self):
         """
-        GET /:login/machines/:id/tags
+        ::
         
-        Returns the complete set of tags for this machine as a dict. A local 
-        copy is not kept because these are essentially search keys.
+            GET /:login/machines/:id/tags
+        
+        :Returns: complete set of tags for this machine
+        :rtype: :py:class:`dict` 
+        
+        A local copy is not kept because these are essentially search keys.
         """
         j, _ = self.datacenter.request('GET', self.path + '/tags')
         return j
     
     def add_tags(self, **kwargs):
         """
-        POST /:login/machines/:id/tags
+        ::
+        
+            POST /:login/machines/:id/tags
         
         Appends the tags (expressed as arbitrary keyword arguments) to those 
-        already set for the Machine.
+        already set for the machine.
         """
         j, _ = self.datacenter.request('POST', self.path + '/tags', 
             params=kwargs)
@@ -321,9 +410,12 @@ class Machine(object):
     
     def get_tag(self, tag):
         """
-        GET /:login/machines/:id/tags/:tag
+        ::
         
-        Returns a the value for a single tag as text.
+            GET /:login/machines/:id/tags/:tag
+        
+        :Returns: the value for a single tag
+        :rtype: :py:class:`basestring`
         """
         headers = {'Accept': 'text/plain'}
         j, _ = self.datacenter.request('GET', self.path + '/tags/' + tag)
@@ -331,25 +423,36 @@ class Machine(object):
     
     def delete_tag(self, tag):
         """
-        DELETE /:login/machines/:id/tags/:tag
+        ::
         
-        Delete a tag and its corresponding value on the Machine.
+            DELETE /:login/machines/:id/tags/:tag
+        
+        Delete a tag and its corresponding value on the machine.
         """
         j, r = self.datacenter.request('DELETE', self.path + '/tags/' + tag)
         r.raise_for_status()
     
     def delete_all_tags(self):
         """
-        DELETE /:login/machines/:id/tags/:tag
+        ::
         
-        Delete all tags and their corresponding values on the Machine.
+            DELETE /:login/machines/:id/tags
+        
+        Delete all tags and their corresponding values on the machine.
         """
         j, r = self.datacenter.request('DELETE', self.path + '/tags')
         r.raise_for_status()
     
     def raw_snapshot_data(self, name):
         """
-        GET /:login/machines/:id/snapshots/:name
+        ::
+        
+            GET /:login/machines/:id/snapshots/:name
+        
+        :param name: identifier for snapshot
+        :type name: :py:class:`basestring`
+        
+        :rtype: :py:class:`dict`
         
         Used internally to get a raw dict of a single machine snapshot.
         """
@@ -359,20 +462,30 @@ class Machine(object):
     
     def snapshots(self):
         """
-        GET /:login/machines/:id/snapshots
+        ::
         
-        Lists all snapshots for the Machine, returning a list of Snapshot 
-        objects.
+            GET /:login/machines/:id/snapshots
+        
+        :rtype: :py:class:`list` of :py:class:`smartdc.machine.Snapshot`
+        
+        Lists all snapshots for the Machine.
         """
         j, _ = self.datacenter.request('GET', self.path + '/snapshots')
         return [Snapshot(machine=self, data=s) for s in j]
     
     def create_snapshot(self, name):
         """
-        POST /:login/machines/:id/snapshots
+        ::
         
-        Create a snapshot for this machine with the given 'name'. Returns a 
-        single Snapshot object.
+            POST /:login/machines/:id/snapshots
+        
+        :param name: identifier for snapshot
+        :type name: :py:class:`basestring`
+        
+        :rtype: :py:class:`smartdc.machine.Snapshot`
+        
+        Create a snapshot for this machine's current state with the given 
+        `name`.
         """
         params = {'name': name}
         j, _ = self.datacenter.request('POST', self.path + '/snapshots', 
@@ -381,9 +494,14 @@ class Machine(object):
     
     def start_from_snapshot(self, name):
         """
-        POST /:login/machines/:id/snapshots/:name
+        ::
         
-        Start the machine from a snapshot with the given 'name'.
+            POST /:login/machines/:id/snapshots/:name
+        
+        :param name: identifier for snapshot
+        :type name: :py:class:`basestring`
+        
+        Start the machine from the snapshot with the given 'name'.
         """
         _, r = self.datacenter.request('POST', self.path + '/snapshots/' + 
             str(name))
@@ -392,10 +510,17 @@ class Machine(object):
     
     def snapshot(self, name):
         """
-        GET /:login/machines/:id/snapshots/:name
+        ::
         
-        Return a Snapshot object that already exists for the machine, 
-        identified by 'name'.
+            GET /:login/machines/:id/snapshots/:name
+        
+        :param name: identifier for snapshot
+        :type name: :py:class:`basestring`
+        
+        :rtype: :py:class:`smartdc.machine.Snapshot`
+        
+        Return a snapshot that already exists for the machine, identified by 
+        `name`.
         """
         return Snapshot(machine=self, name=name)
     
@@ -404,19 +529,42 @@ class Snapshot(object):
     """
     A local proxy representing the current state of a machine snapshot.
     
-    A Snapshot object is intended to be a convenient container for a 
-    snapshot's state and for performing methods on it.
+    A :py:class:`smartdc.machine.Snapshot` object is intended to be a 
+    convenient container for a snapshot's state and for performing methods on 
+    it.
     """
     def __init__(self, machine, name=None, data=None):
         """
-        Typically, a snapshot object is instantiated automatically by 
-        creating or listing snapshots for a Machine. However, a snapshot may
-        be manually instantiated by giving a 'machine' and a 'name' as 
+        :param machine: source of the snapshot
+        :type machine: :py:class:`smartdc.machine.Machine`
+        
+        :param name: identifier for snapshot
+        :type name: :py:class:`basestring`
+        
+        :param data: raw data for instantiation
+        :type data: :py:class:`dict`
+        
+        Typically, a :py:class:`smartdc.machine.Snapshot` object is 
+        instantiated automatically by creating or listing snapshots for a 
+        :py:class:`smartdc.machine.Machine`. However, a snapshot may be 
+        manually instantiated by giving a 'machine' and a 'name' as 
         parameters, and the snapshot's state and other data will be queried
         from the server.
+        
+        The following attributes are read-only:
+        
+        :var name: human-readable label for the machine
+        :var state: last-known state of the machine
+        :var created: :py:class:`datetime.datetime` of machine creation 
+            time
+        :var updated: :py:class:`datetime.datetime` of machine update 
+            time
+        
         """
         self.name = name or data.pop('name')
         self.machine = machine
+        """:py:class:`smartdc.machine.Machine` object that contains this 
+        snapshot"""
         if not data:
             data = self.machine.raw_snapshot_data(self.name)
         self._save(data)
@@ -453,9 +601,11 @@ class Snapshot(object):
     
     def refresh(self):
         """
-        GET /:login/machines/:id/snapshots/:name
+        ::
         
-        Fetch the existing state and values for the Snapshot 
+            GET /:login/machines/:id/snapshots/:name
+        
+        Fetch the existing state and values for the snapshot 
         and commit the values locally.
         """
         data = self.machine.raw_snapshot_data(self.name)
@@ -463,17 +613,24 @@ class Snapshot(object):
     
     def status(self):
         """
-        GET /:login/machines/:id/snapshots/:name
+        ::
+        
+            GET /:login/machines/:id/snapshots/:name
+        
+        :Returns: the current state
+        :rtype: :py:class:`basestring`
         
         Refresh the snapshot's information by fetching it remotely, then 
-        returning the state as a string.
+        returning the :py:attr:`state` as a string.
         """
         self.refresh()
         return self.state
     
     def delete(self):
         """
-        DELETE /:login/machines/:id/snapshots/:name
+        ::
+        
+            DELETE /:login/machines/:id/snapshots/:name
         
         Deletes the snapshot from the machine.
         """
@@ -482,7 +639,11 @@ class Snapshot(object):
     
     def start(self):
         """
-        POST /:login/machines/:id/snapshots/:name
+        ::
+        
+            POST /:login/machines/:id/snapshots/:name
+        
+        :Returns: self
         
         Initiate boot of the machine from this snapshot.
         """
